function [handle] = colourbar(location,range,interval,p1,v1,p2,v2,p3,v3,p4,v4,p5,v5)
%COLOURBAR Plot a colorbar to give the scale of a 'pcolor' type plot.
%   CRFcolourbar(location,range,interval,param1,value1,param2,value2,...)
%
%   prints a nice colourbar on your figure at `location' (in normalised figure units)
%   `range' is a two-element vector containing the extremes of the range
%   `interval' is the colour interval
%   parameter and value pairs follow and allow for more customisation:
%   For example:
%      `orientation','horizontal'
%      `orientation','vertical'   (default)
%       `ends','pointed'
%      `ends','square'   (default)
%      `fontsize',12
%      `scale','log'   (in this case `range' and `interval' refer to log10 of the value)
%      `scale','linear'   (default)
%      `cmap','plus'   (for when you use pcolorplus and have greys in the colourmap)
%

horizontal=0;
pointed=0;
logarithmic=0;
cmapplus=0;
fontsize=10;
if nargin > 3
   for loop = 1:(nargin-2)/2
      eval(['parameter=p' int2str(loop) ';']);
      eval(['value=v' int2str(loop) ';']);
      if strcmp(upper(parameter),'ORIENTATION')
         if strcmp(upper(value),'HORIZONTAL'),horizontal=1;end
      elseif strcmp(upper(parameter),'ENDS')
         if strcmp(upper(value),'POINTED'),pointed=1;end
      elseif strcmp(upper(parameter),'FONTSIZE')
         fontsize=value;
      elseif strcmp(upper(parameter),'SCALE')
         if strcmp(upper(value),'LOG')
            logarithmic=1;
         end
      elseif strcmp(upper(parameter),'CMAP')
         if strcmp(upper(value),'PLUS')
            cmapplus=1;
         end
      else
         error(['Unknown command parameter: ' parameter])
      end
   end
end
handle=axes('position',location,'FontSize',fontsize);
if diff(range) > 0
  D = [range(1):interval:range(2)+interval];
  C=D'*[1 1];
else
  D = [range(2):interval:range(1)+interval];
  range = [range(2) range(1)];
  C = fliplr(D)'*[1 1]-interval;
end
cmap=colormap;
cmapsize=size(cmap,1);

if horizontal
   if logarithmic
      axis([10.^range 0 1])
      set(gca,'XScale','log')
      set(gca,'YTickLabel',[' ']);
      axes('position',location,'FontSize',fontsize)
   end
   pcolor(D',[0 1],C')
   set(gca,'YTickLabel',[' '],'ytick',[0 1]);
   if cmapplus
      caxis([range(1)-diff(range)/(cmapsize-2) range(2)])
   else
      caxis(range)
   end
   axis([range 0 1])
   if logarithmic
%      set(gca,'Visible','off')
      set(gca,'XTickLabel',[],'YTickLabel',[])
   end
   if pointed
      hold on
      patch([range(1) range(1) range(1)-interval],[0 1 0.5],cmap(1+cmapplus,:),'CData',cmap(1+cmapplus,:),'LineWidth',[0.2],'EdgeColor',[0 0 0],'Clipping','off');
      patch([range(2) range(2) range(2)+interval],[0 1 0.5],cmap(cmapsize,:),'CData',cmap(cmapsize,:),'LineWidth',[0.2],'EdgeColor',[0 0 0],'Clipping','off');
      hold off
   end

else
   if logarithmic
      axis([0 1 10.^range])
      set(gca,'YScale','log')
      set(gca,'XTickLabel',[' ']);
      axes('position',location,'FontSize',fontsize)
   end
   pcolor([0 1],D',C)
   set(gca,'XTickLabel',[' '],'xtick',[0 1]);
   if cmapplus
      caxis([range(1)-diff(range)/(cmapsize-2) range(2)])
   else
      caxis(range)
   end
   axis([0 1 range])
   if logarithmic
%      set(gca,'Visible','off')
      set(gca,'XTickLabel',[],'YTickLabel',[])
   end
   if pointed
      hold on
      patch([0 1 0.5],[range(1) range(1) range(1)-interval],cmap(1+cmapplus,:),'CData',cmap(1+cmapplus,:),'LineWidth',[0.2],'EdgeColor',[1 1 1],'Clipping','off');
      patch([0 1 0.5],[range(2) range(2) range(2)+interval],cmap(cmapsize,:),'CData',cmap(cmapsize,:),'LineWidth',[0.2],'EdgeColor',[1 1 1],'Clipping','off');
      hold off
   end
end
set(gca,'layer','top');





