function [data, attribute, beta_bias] = load_lidar(thedate, site, parameter, instrument, names);
%
% [data, attribute, beta_bias] = load_lidar(day, site, parameter);
%    Load lidar data for the day and site via the structures
%    'data' and 'attribute'. 'parameter' is optional; if it
%    has the value 'test' then a 1 will be returned in 'data'
%    if the data are available, 0 otherwise. beta_bias contains the
%    estimated calibration accuracy in dB.
%

is_a_test = 0;
data = [];
attribute = [];
beta_bias = 3;

if nargin < 1
  help load_lidar;
  return;
end

if nargin < 2
  site = 'chilbolton';
elseif strcmp(lower(site),'arm')
  site = 'arm-sgp';
end

if isnumeric(thedate)
  thedate = num2str(thedate);
end

if nargin > 2
  if strcmp(parameter, 'test')
    is_a_test = 1;
    data = 0;
  end
else
  parameter = '';
end

if nargin < 4 
  instrument = '';
end

if strcmp(lower(site),'reading')
  site = 'actual';
end

if isempty(instrument) 
  switch lower(site)
   case 'chilbolton'
    instrument = 'ct75k';
   case {'palaiseau','sirta'}
    if str2double(thedate) < 20090101
      instrument = 'ld40';
    else
      instrument = 'als450';
    end    
   case {'lindenberg','falkenberg'}
    instrument = 'ld40';
   case 'eureka'
    instrument = 'ahsrl';
   case 'hornisgrinde'
    instrument = 'imktro-doppler-lidar';
   case 'vocals'
    instrument = 'hrdl';
   case {'mace_head','exeter','camborne'}
    instrument = 'chm15k';
   case {'leipzig'};
    instrument = 'chm15x';
   case {'potenza','kenttarova','juelich','summit'}
    instrument = 'ct25k';
   case {'vanta','kumpula'}
    instrument = 'cl31';    
   case {'actual','cardington','ganges_valley'}
    instrument = 'halo-doppler-lidar';   
   otherwise
    if strncmp(lower(site),'arm',3)
      instrument = 'ct25k';
    end
  end
end

if strcmp(lower(site),'arm-sgp') & str2double(thedate) < 20000522 & strcmp(instrument,'ct25k')
  instrument = 'blc';
end 

if strncmp(lower(site),'arm',3) & str2double(thedate) < 19990000
  instrument = 'mpl';
end

if strcmp(lower(site),'chilbolton')
  switch instrument
   case {'dopplerlidar','doppler-lidar','halodopplerlidar','lidar-doppler-halo'}
    instrument = 'halo-doppler-lidar';
   case {'lidar-leosphere','leosphere-lidar','ez-lidar'}
    instrument = 'ezlidar';
   case {'chm15k'}
    instrument = 'chm15k';
  end
end  

if strcmp(lower(site),'palaiseau')
  switch instrument
   case {'lidar-leosphere','leosphere-lidar','ez-lidar','ezlidar'}
    instrument = 'als450';
  end
end

if any(strcmp(lower(site),{'malmi','repartee','actual','cardington','mace_head','ganges_valley','san-jose','kumpula','kuopio','uto','hyytiala'}))
  switch instrument
   case {'dopplerlidar','doppler-lidar','halodopplerlidar','lidar-doppler-halo'}
    instrument = 'halo-doppler-lidar';
  end
end

if strncmp(lower(site),'arm',3)
  switch instrument
   case {'dopplerlidar','doppler-lidar','halodopplerlidar','lidar-doppler-halo'}
    instrument = 'halo-doppler-lidar';
   case {'lidar-leosphere','leosphere-lidar','ez-lidar'}
    instrument = 'ezlidar';
  end
end  

if nargin < 5
  names = '';
end

switch lower(site)
 case 'chilbolton'
  switch instrument
   case 'ct75k'
    file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) thedate  '_chilbolton-ct75k.nc'];
   case 'halo-doppler-lidar'
    %  file_name = ['/home/radar/data/chilbolton/calibrated/' instrument
    %  '/' thedate(1:4) '/' thedate  '_chilbolton-' instrument '.nc'];
%    if str2double(thedate) < 20100400
%      file_name = [get_dir_paths('instrument', lower(site), 'uncalibrated', instrument, thedate(1:4)) thedate  '_chilbolton_' instrument '_uc.nc'];
%    else
      file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) thedate  '_chilbolton_' instrument '.nc'];
%    end
   case {'ezlidar','chm15k'}
    file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) thedate  '_chilbolton_' instrument '.nc'];
  end
 case 'cabauw'
  file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) thedate  '_cabauw-ct75k.nc'];
 case {'palaiseau','sirta'}
  switch lower(instrument)
   case 'als450'
    file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) thedate  '_palaiseau_' instrument '.nc'];
   case 'cl31'
    file_name = [get_dir_paths('instrument', lower(site), 'uncalibrated', instrument, thedate(1:4)) thedate  '_palaiseau_' instrument '.nc'];

   otherwise
    file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) thedate  '_sirta-' instrument '.nc'];
  end
 case {'ganges_valley','lindenberg','falkenberg','arm-sgp','arm-nsa','arm-nauru','arm-manus','arm-darwin','arm-niamey','arm-murgtal','arm-graciosa','arm-maldives','arm-shouxian','arm-cape-cod','hornisgrinde','exeter','malmi','vanta','leipzig','actual','san-jose','summit'} 
  file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) thedate '_' lower(site) '_' instrument '.nc'];
 case 'kumpula'
  if strcmp(lower(instrument),'halo-doppler-lidar')
    file_name = [get_dir_paths('instrument', lower(site), 'uncalibrated', [instrument '-34-co'], thedate(1:4)) thedate '_fmi_' instrument '-34-co_uc.nc'];
  else
    file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) thedate '_' lower(site) '_' instrument '.nc'];
  end
 case 'mace_head'
  if strcmp(lower(instrument),'halo-doppler-lidar')
    file_name = [get_dir_paths('instrument', lower(site), 'uncalibrated', instrument, thedate(1:4)) thedate '_fmi_' instrument '-34-co_uc.nc'];
  else
    file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) thedate '_' lower(site) '_' instrument '.nc'];
  end
 case 'repartee'
  file_name = [get_dir_paths('instrument', lower(site), 'uncalibrated', instrument, thedate(1:4)) thedate  '_' lower(site) '_' instrument '.nc'];
 case {'reading','cardington'}
  file_name = [get_dir_paths('instrument', lower(site), 'uncalibrated', instrument, thedate(1:4)) thedate  '_' lower(site) '_' instrument '_uc.nc'];
 case 'eureka'
  file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) instrument '_' thedate  '*.nc'];
  file_name = deblank(ls(file_name));
 case {'camborne','kenttarova'}
  file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) thedate '_' lower(site) '*.nc'];
  file_name = deblank(ls(file_name));
 case 'potenza'
  file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) 'CNR-IMAA-Ceilometer_TitoScalo_' thedate '_*.nc'];
  file_name = deblank(ls(file_name));
 case 'juelich'
%  file_name = [get_dir_paths('instrument', lower(site), 'uncalibrated', instrument, thedate(1:4)) thedate(3:8)  '_' instrument '_jue_l0b.nc'];
  file_name = [get_dir_paths('instrument', lower(site), 'uncalibrated', instrument, thedate(1:4)) thedate '_' lower(site) '_' instrument '_uc.nc'];
 case 'vocals'
  file_name = [get_dir_paths('instrument', lower(site), 'calibrated', instrument, thedate(1:4)) thedate  '_' lower(site) '_' instrument '.nc'];
 otherwise
  warning(['Site ' site ' unrecognised']);
  return;
end

if ~exist(file_name,'file')
  warning([file_name ' not found']);
  return;
elseif is_a_test
  data = 1;
  return;
end
  
max_height = 14;

switch lower(site)
 case {'chilbolton','repartee','lindenberg','falkenberg','camborne','malmi','actual','cardington'}
  
  if ~isempty(names)
    [data, attribute] = load_nc_struct(file_name, [names, {'scale_factor','altitude'}]);
  else
    [data, attribute] = load_nc_struct(file_name);
  end

  if ~strcmp(lower(instrument),'chm15k')  
    switch lower(site)
     case 'chilbolton'
      % If scale_factor is not the default system value of 1.e-7 then
      % assume that the lidar has been calibrated using Ewan O'Connor's
      % method, for which the error is assumed to be around 7%, or 0.3
      % dB.
      if isfield(attribute.beta, 'scale_factor')
        if attribute.beta.scale_factor ~= 1.e-7
          beta_bias = 0.3;
        end
      end
      
      if ~isfield(attribute.global, 'history')
        attribute.global.history = 'unknown';
      end
      if strcmp(instrument,'ct75k') & ~isfield(attribute.global, 'source')
        attribute.global.source = 'Chilbolton Vaisala 905-nm CT75K lidar ceilometer';
      end
      
      if strcmp(instrument,'ct75k')
        data.lidar_beam_divergence = 0.75; 
        attribute.lidar_beam_divergence.long_name = 'Lidar laser beam divergence';
        attribute.lidar_beam_divergence.units = 'mrad'; 
        
        data.lidar_telescope_field_of_view = 0.66;
        attribute.lidar_telescope_field_of_view.long_name = 'Lidar telescope field of view';
        attribute.lidar_telescope_field_of_view.units = 'mrad';
      end
      if strcmp(instrument,'ezlidar')
        % These values are approximate, taken from: 
        % 'Lidar observations by circling the London orbital motorway'
        % by J.-C. Raut, P. Chazette, J. Haywood and P. Royer
        %  Proceedings of the 8th International Symposium on Tropospheric
        %  Profiling, ISBN 978-90-6960-233-2, Delft, The Netherlands, October 2009. 
        data.lidar_beam_divergence = 0.2; 
        attribute.lidar_beam_divergence.long_name = 'Lidar laser beam divergence';
        attribute.lidar_beam_divergence.units = 'mrad'; 
        
        data.lidar_telescope_field_of_view = 2;
        attribute.lidar_telescope_field_of_view.long_name = 'Lidar telescope field of view';
        attribute.lidar_telescope_field_of_view.units = 'mrad';
      end
     case {'lindenberg','falkenberg'}
      % Check that time axis is distinct (drop rays if not)
      index=find(diff(data.time)<=0);
      while ~isempty(index)
        data.time(index)=[];
        data.beta(index,:)=[];
        if isfield(data, 'beta_raw')
          data.beta_raw(index,:)=[];
        end
        if isfield(data, 'mean_noise')
          data.mean_noise(index)=[];
        end
        if isfield(data, 'base1')
          data.base1(index)=[];
        end
        if isfield(data, 'base2')
          data.base2(index)=[];
        end
        if isfield(data, 'base3')
          data.base3(index)=[];
        end
        index=find(diff(data.time)<=0);
      end
      
      % clean data
      data.beta (data.beta < 1e-7) = NaN;
      
      if max(data.range) < 12000
        % add fictitious gates up to height of radar
        max_height = 14000;
        
        dz = median(diff(data.range));
        start_gate = length(data.range) +1;
        last_gate = start_gate + ceil(abs((max_height - max(data.range)) ./ dz));
        
        data.range = [data.range; data.range(end) + [1:(last_gate-start_gate)]' .* dz];
        data.beta = [data.beta ones(length(data.time),last_gate-start_gate).*NaN];
        data.beta_raw = [data.beta_raw ones(length(data.time),last_gate-start_gate).*NaN];
      end  
      
      if ~isfield(data, 'laser_beam_divergence')
        data.laser_beam_divergence = 1;
        attribute.laser_beam_divergence.long_name = 'Lidar laser beam divergence';
        attribute.laser_beam_divergence.units = 'mrad';
        
        data.telescope_field_of_view = 1.2;
        attribute.telescope_field_of_view.long_name = 'Lidar telescope field of view';
        attribute.telescope_field_of_view.units = 'mrad';
      end
    end
  end
  if isfield(data,'wavelength')
    data.wavelength = data.wavelength(1);
  end
  if median(diff(data.time)).*3600 < 20 & ~(any(strcmp(lower(site),{'repartee','cardington','malmi','reading'})))
    % average 2 rays
    time = zeros(floor(length(data.time)./2),1);
    beta = zeros(floor(length(data.time)./2),length(data.range)).*nan;
    for ii = 1:length(time)
      time(ii) = mean(data.time((2*ii)-1:2.*ii));
      beta(ii,:) = nanmean(data.beta((2*ii)-1:2.*ii,:));
    end
    data.time_orig = data.time;
    data.time = time;
    data.beta_orig = data.beta;
    data.beta = beta;
    attribute.global.history = ['Original data has been averaged to 30 seconds.'];
  end
 case 'cabauw'
  if ~isempty(names)
    [data, attribute] = load_nc_struct(file_name, [names, {'scale_factor'}]);
  else
    [data, attribute] = load_nc_struct(file_name);
  end
  
  % Assume calibration is about 2.6e-7 with an error of about 15%, or
  % 0.6 dB
  if isfield(attribute.beta, 'scale_factor')
    attribute.beta.scale_factor = 2.6e-7;
    beta_bias = 0.6;
  end

  if ~isfield(attribute.global, 'history')
    attribute.global.history = 'unknown';
  end
  if ~isfield(attribute.global, 'source')
    attribute.global.source = 'Cabauw Vaisala 905-nm CT75K lidar ceilometer';
  end
  
  % Check that time axis is distinct (drop rays if not)
  index=find(diff(data.time)<=0);
  if ~isempty(index)
    data.time(index+1)=[];
    data.beta(index+1,:)=[];
    if isfield(data, 'base1')
      data.base1(index+1)=[];
    end
    if isfield(data, 'base2')
      data.base2(index+1)=[];
    end
    if isfield(data, 'base3')
      data.base3(index+1)=[];
    end
  end
    
  data.lidar_beam_divergence = 0.75; 
  attribute.lidar_beam_divergence.long_name = 'Lidar laser beam divergence';
  attribute.lidar_beam_divergence.units = 'mrad';

  data.lidar_telescope_field_of_view = 0.66;
  attribute.lidar_telescope_field_of_view.long_name = 'Lidar telescope field of view';
  attribute.lidar_telescope_field_of_view.units = 'mrad';
  
 case {'palaiseau','sirta'}
  if ~isempty(names)
    [data, attribute] = load_nc_struct(file_name, names);
  else
    [data, attribute] = load_nc_struct(file_name);
  end
  
  if ~isfield(attribute.global, 'history')
    attribute.global.history = 'unknown';
  end
  if ~isfield(attribute.global, 'source')
    attribute.global.source = 'unknown';
  end
  
  switch instrument
   case 'als450'
  % Assume calibration has an error of about 15%, or 0.6 dB
  beta_bias = 0.6; 
  
  % Don't know what these are yet
  data.lidar_beam_divergence = -999; 
  attribute.lidar_beam_divergence.long_name = 'Lidar laser beam divergence';
  attribute.lidar_beam_divergence.units = 'mrad';
  
  data.lidar_telescope_field_of_view = -999;
  attribute.lidar_telescope_field_of_view.long_name = 'Lidar telescope field of view';
  attribute.lidar_telescope_field_of_view.units = 'mrad';
    
   case 'ld40'
    beta_bias = 1.1;    
    
    % Check that time axis is distinct (drop rays if not)
    index=find(diff(data.time)<=0);
    if ~isempty(index)
      data.time(index)=[];
      data.beta(index,:)=[];
      if isfield(data, 'base1')
	data.base1(index)=[];
      end
      if isfield(data, 'base2')
	data.base2(index)=[];
      end
      if isfield(data, 'base3')
	data.base3(index)=[];
      end
    end
  
    data.lidar_beam_divergence = 1; 
    attribute.lidar_beam_divergence.long_name = 'Lidar laser beam divergence';
    attribute.lidar_beam_divergence.units = 'mrad'; 
    
    data.lidar_telescope_field_of_view = 1.2;
    attribute.lidar_telescope_field_of_view.long_name = 'Lidar telescope field of view';
    attribute.lidar_telescope_field_of_view.units = 'mrad';
    
   otherwise  % lna
    data.altitude=156;
    data.wavelength=532;
    data.latitude=2.208;
    data.longitude=48.173;
    %    data.elev=85; ?

    data.range=data.range.*1e-3;
    
    index=find(data.time<0.000001|data.pr2(:,1)<-990);
    data.time(index)=[];
    data.pr2(index,:)=[];
      
    % Check that time axis is distinct (drop rays if not)
    index=find(diff(data.time)<=0);
    if ~isempty(index)
      data.time(index)=[];
      data.pr2(index,:)=[];
      if isfield(data, 'base1')
	data.base1(index)=[];
      end
      if isfield(data, 'base2')
	data.base2(index)=[];
      end
      if isfield(data, 'base3')
	data.base3(index)=[];
      end
    end
  
    %dodgy data (20040109)
    if strcmp(thedate,'20040109')
      index=find(max(data.pr2(:,1:30)')<0.0002);
      data.pr2(index,:)=[];
      data.time(index)=[];
    end
    data.beta_raw=data.pr2.*1.4e-5; % initial calibration
   
    
    %calculate noise floor from top 50 gates and clean data
    data.noisefloor = 2 .* mean( data.pr2(:,end-50:end)./((ones(length(data.time),1)*data.range(end-50:end)').^2),2) + 5 * std( data.pr2(:,end-50:end)./((ones(length(data.time),1)*data.range(end-50:end)').^2),[],2) ;
    
    data.beta_raw( ((data.pr2))./(ones(length(data.time),1)*data.range').^2 < data.noisefloor * ones(1,length(data.range)) ) = nan;

    %recalibration - calibration varies so use noisefloor as guide
    %for threshold.
    %first find gaps in time (when changes to setup have been made)
    index = [0; find(diff(data.time)>5.*median(diff(data.time))); length(data.time)];

    
    if strcmp(thedate,'20040204')|strcmp(thedate,'20040203')
      threshold = 3e-4;
      calfactor = 4;
%    elseif strcmp(thedate,'20040203')
%      threshold = 3e-4;
%      calfactor = 4;
    else      
      threshold = 2e-4;
      calfactor = 10;
    end
    
    for ii=2:length(index)
      if median(data.noisefloor(index(ii-1)+1:index(ii)))<threshold
	data.beta_raw(index(ii-1)+1:index(ii),:)=data.beta_raw(index(ii-1)+1:index(ii),:).*calfactor;
      end
    end
    data.beta=data.beta_raw;
  
    %calculate approximate molecular (rayleigh) backscatter and remove
    %use model data when available
    mol_return=interp1([0:0.001:20],molecular_scat([0:0.001:20],532),data.range);
    data.beta(data.beta_raw < ones(length(data.time),1)*mol_return') = nan;
    
    %despeckle
    index = despeckle(data.beta);
    data.beta = remove_inner_pixels(data.beta, index,'despeckle');
    
    %despeckle
    index = despeckle(data.beta);
    data.beta = remove_inner_pixels(data.beta, index,'despeckle');
    
    % Write attributes
    attribute.beta.units = 'sr-1 m-1' ;
    attribute.beta.units_html = 'sr<sup>-1</sup> m<sup>-1</sup>' ;
    attribute.beta.long_name = 'Attenuated backscatter coefficient' ;
    attribute.beta.missing_value = 0 ;
    attribute.beta.FillValue_ = 0 ;
    
    attribute.latitude.units = 'degrees_north';
    attribute.latitude.long_name = 'Latitude of lidar';
    attribute.latitude.standard_name = 'latitude';
    
    attribute.longitude.units = 'degrees_east';
    attribute.longitude.long_name = 'Longitude of lidar';
    attribute.longitude.standard_name = 'longitude';
    
    attribute.altitude.units = 'm';
    attribute.altitude.long_name = 'Height of instrument above mean sea level';
    
    attribute.elev.units = 'degrees';
    attribute.elev.long_name = 'Elevation above horizon';
    attribute.elev.missing_value = -999;
    attribute.elev.FillValue_ = -999;
        
    attribute.beta.comment = 'This variable should be multiplied by the value in the scale_factor attribute. Speckle noise has been removed, but note that when speckle noise is strong its removal reduces the effective sensitivity of the instrument.' ;
    attribute.wavelength.units = 'nm' ;
    attribute.wavelength.long_name = 'Lidar wavelength';
    
    % Don't know what these are yet
    data.lidar_beam_divergence = -999; 
    attribute.lidar_beam_divergence.long_name = 'Lidar laser beam divergence';
    attribute.lidar_beam_divergence.units = 'mrad';
    
    data.lidar_telescope_field_of_view = -999;
    attribute.lidar_telescope_field_of_view.long_name = 'Lidar telescope field of view';
    attribute.lidar_telescope_field_of_view.units = 'mrad';
  end
  
  % Check that range is in km
  if (data.range(2)-data.range(1)) > 1 
    data.range = data.range./1e3;
    attribute.range.units = 'km';
  end
  
  
 
 case {'arm-nsa','arm-nauru','arm-manus','arm-darwin','arm-sgp','arm-niamey','arm-murgtal','arm-graciosa','arm-maldives','arm-shouxian','arm-cape-cod','ganges_valley','summit'}
  
  if ~isempty(names)
    [data, attribute] = load_nc_struct(file_name, [names, {'scale_factor','altitude'}]);
  else
    [data, attribute] = load_nc_struct(file_name);
  end
  
  % If scale_factor is not the default system value of 1.e-7 then
  % assume that the lidar has been calibrated using Ewan O'Connor's
  % method, for which the error is assumed to be around 7%, or 0.3
  % dB.
  if isfield(attribute.beta, 'scale_factor')
    if attribute.beta.scale_factor ~= 1.e-7
      beta_bias = 0.3;
    end
  else
    beta_bias = 1; % error is about 25%    
  end
  
  % average 2 rays
  if strcmp(instrument,'ct25k')
    time = zeros(floor(length(data.time)./2),1);
    beta = zeros(floor(length(data.time)./2),length(data.range)).*nan;
    beta_raw = zeros(floor(length(data.time)./2),length(data.range)).*nan;
    for ii = 1:length(time)
      time(ii) = mean(data.time((2*ii)-1:2.*ii));
      beta(ii,:) = nanmean(data.beta((2*ii)-1:2.*ii,:));
      beta_raw(ii,:) = mean(data.beta_raw((2*ii)-1:2.*ii,:));
    end    
  
    data.time_orig = data.time;
    data.time = time;
    data.beta_orig = data.beta;
    data.beta = beta;
    data.beta_raw_orig = data.beta_raw;
    data.beta_raw = beta_raw;

    attribute.global.history = ['Original data has been averaged to 30 seconds.'];
  end
  
  if strcmp(instrument,'mpl') &  strcmp(lower(site),'arm-sgp')
    data.beta(:,1:3) = NaN;    
  end
  % add fictitious gates up to height of radar
  switch lower(site)
   case {'arm-nsa','summit'}
    max_height = 14;
   case {'arm-murgtal'}
    max_height = 16;
   case {'arm-sgp','arm-graciosa','arm-cape-cod'}
    max_height = 18;
   case {'arm-nauru','arm-manus','arm-darwin','arm-niamey','arm-maldives'}
    max_height = 21;
  end

  if max(data.range) < max_height 
    dz = median(diff(data.range));
    start_gate = length(data.range) +1;
    last_gate = start_gate + ceil(abs((max_height - max(data.range)) ./ dz));
    
    data.range = [data.range; data.range(end) + [1:(last_gate-start_gate)]' .* dz];
    data.beta = [data.beta ones(length(data.time),last_gate-start_gate).*NaN];
    data.beta_raw = [data.beta_raw ones(length(data.time),last_gate-start_gate).*NaN];

    if isfield(data,'time_orig')
      data.beta_raw_orig = [data.beta_raw_orig ones(length(data.time_orig),last_gate-start_gate).*NaN];
    end
    if isfield(data,'molecular')
      data.molecular = [data.molecular ones(length(data.time),last_gate-start_gate).*NaN];
    end
    if isfield(data,'depolarization')
      data.depolarization = [data.depolarization ones(length(data.time),last_gate-start_gate).*NaN];
    end
    if isfield(data,'beta_raw_cr')
      data.beta_raw_cr = [data.beta_raw_cr ones(length(data.time),last_gate-start_gate).*NaN];
    end
    if isfield(data,'flag')
      data.flag = [data.flag ones(length(data.time),last_gate-start_gate).*NaN];
    end
    if isfield(data,'v')
      data.v = [data.v ones(length(data.time),last_gate-start_gate).*NaN];
    end
    if isfield(data,'v_raw')
      data.v_raw = [data.v_raw ones(length(data.time),last_gate-start_gate).*NaN];
    end
    if isfield(data,'signal')
      data.signal = [data.signal ones(length(data.time),last_gate-start_gate).*NaN];
    end
    if isfield(data,'SNR')
      data.SNR = [data.SNR ones(length(data.time),last_gate-start_gate).*NaN];
    end
    if isfield(data,'SNR_cr')
      data.SNR_cr = [data.SNR_cr ones(length(data.time),last_gate-start_gate).*NaN];
    end
  end

  % if MPL, make sure flag is in usable format!
  if isfield(data,'flag')
    data.flag = double(data.flag);    
    % improve flag! for arm-murgtal
    if strcmp(lower(site),'arm-murgtal') 
      
      flag = ones(size(data.flag));
      flag(data.flag~=7)=0;
      index = find(runmean(flag,7,1)<0.4 & data.flag==7);
      data.flag(index) = 6;
      index = find(data.flag==7);

      beta = data.beta*NaN;
      beta(index) = data.beta(index);
      beta = nanrunmean(beta,5,1);
      depol = data.depolarization*NaN;
      depol(index) = data.depolarization(index);
      depol = nanrunmean(depol,5,1);
      index = find(depol > 0.1 & beta > 2e-7);
      data.flag(index) = 2;
      index = find(depol > 0.1 & beta < 2e-7);      
      data.flag(index) = 0;
      index = find(depol > 0.01 & depol < 0.1 & beta > 2e-7);
      data.flag(index) = 1;
      index = find(data.flag==7);
      data.flag(index) = 6;
      
%      index = find(~isfinite(depol)& beta < 5e-7);
%      length(index)
%      index = find(data.flag==7);
%      length(index)
      
      flag = data.flag;    
      flag(flag==6)=0;
      flag(flag>1)=1; 
      index = find(runmean(flag,7,1)<0.4 & data.flag~=6);
      data.flag(index) = 0;
      flag(data.flag~=2)=0;
      % Final Clean
      index = find(data.flag<1);
      data.beta(index) = NaN;
      data.depolarization(index) = NaN;
    end
  end
  
  % Check that time axis is distinct (drop rays if not)
  index=find(diff(data.time)<=0);
  if ~isempty(index)
    data.time(index)=[];
    data.beta(index,:)=[];
    if isfield(data, 'base1')
      data.base1(index)=[];
    end
    if isfield(data, 'base2')
      data.base2(index)=[];
    end
    if isfield(data, 'base3')
      data.base3(index)=[];
    end
    if isfield(data,'molecular')
      data.molecular(index,:) = [];
    end
    if isfield(data,'flag')
      data.flag(index,:) = [];
    end
  end
 
 case {'eureka','hornisgrinde','mace_head','exeter','vanta','kenttarova','juelich','kumpula','vocals','leipzig','san-jose'}
  if ~isempty(names)
    [data, attribute] = load_nc_struct(file_name, names);
  else
    [data, attribute] = load_nc_struct(file_name);
  end
 
  switch lower(site)
   case {'juelich'}
    % add fictitious gates up to height of radar
    max_height = 15;
    if max(data.range) < max_height
      dz = median(diff(data.range));
      newrange = data.range(end)+dz:dz:max_height;
      data.range = [data.range; newrange(:)];
      padnan = zeros(length(data.time),length(newrange)) .* NaN;
      data.beta = [data.beta padnan];
    end
   case {'vocals'}
    data.altitude_raw = data.altitude;
    data.altitude = nanmean(data.altitude);
  end
  
  
 case 'potenza' 
  [data, attribute] = load_nc_struct(file_name);

  beta_bias = 1; % error is about 25%
  data.beta = data.Backscatter .* 2.5e-7;
%%%%%%%%%%%%%%%%%%%%
% need a combination of both r2 and flat background..
  noise = sqrt(abs(data.Background_Light-4) * (data.Height'.^2) ./1e6);
  noise = (std(data.Backscatter(:,end-20:end)'*5))' * ones(1,length(data.Height));
  % first negative point filter

  for ii = 1:length(data.Time);
    index = find(data.Backscatter(ii,:)<0); 
    if ~isempty(index) 
      noise(ii,1:index(1)-1) = 0;
    end
  end

  data.beta(data.Backscatter <= 1 | data.Backscatter < noise.*1.25 ) = NaN;  

  %despeckle
  index = despeckle(data.beta);
  data.beta = remove_inner_pixels(data.beta, index,'despeckle');

  starttime = sscanf(attribute.global.Measurement_Start_Time,'%2d:%2d:%2d');
  data.time = (data.Time + sum(starttime .* [3600; 60; 1])) ./ 3600;
  data.range = data.Height;

  % average 2 rays
  time = zeros(floor(length(data.time)./2),1);
  beta = zeros(floor(length(data.time)./2),length(data.range)).*nan;
  for ii = 1:length(time)
    time(ii) = mean(data.time((2*ii)-1:2.*ii));
    beta(ii,:) = nanmean(data.beta((2*ii)-1:2.*ii,:));
  end
  data.time_orig = data.time;
  data.time = time;
  data.beta = beta;

  %despeckle
  index = despeckle(data.beta);
  data.beta = remove_inner_pixels(data.beta, index,'despeckle');

  % add fictitious gates up to height of radar
  max_height = 15000;
  if max(data.range) < max_height
    dz = median(diff(data.range));
    newrange = data.range(end)+dz:dz:max_height;
    data.range = [data.range; newrange(:)];
    padnan = zeros(length(data.time),length(newrange)) .* NaN;
    data.beta = [data.beta padnan];
  end

  data.latitude = 40.6;
  data.longitude = 15.72;
  data.altitude = 760;
  data.elev = 90 - data.Tilt_Angle;
  data.wavelength = 905;
  data.lidar_beam_divergence = 0.75;
  data.lidar_telescope_field_of_view = 0.66;

  % Write attributes
  attribute.latitude.units = 'degrees_north';
  attribute.latitude.long_name = 'Latitude of antenna';
  attribute.latitude.standard_name = 'latitude';

  attribute.longitude.units = 'degrees_east';
  attribute.longitude.long_name = 'Longitude of antenna';
  attribute.longitude.standard_name = 'longitude';

  attribute.altitude.units = 'm';
  attribute.altitude.long_name = 'Height of instrument above mean sea level';

  attribute.elev.units = 'degrees';
  attribute.elev.long_name = 'Elevation above horizon';
  attribute.elev.missing_value = -999;
  attribute.elev.FillValue_ = -999;

  attribute.wavelength.units = 'nm' ;
  attribute.wavelength.long_name = 'Lidar wavelength' ;

  attribute.lidar_beam_divergence.long_name = 'Lidar laser beam divergence';
  attribute.lidar_beam_divergence.units = 'mrad';
  attribute.lidar_telescope_field_of_view.long_name = 'Lidar telescope field of view';
  attribute.lidar_telescope_field_of_view.units = 'mrad';

  attribute.time.standard_name = 'time';
  attribute.time.long_name = 'Decimal hours since midnight UTC';
  attribute.time.units = ['Hours since 20' thedate(1:4) '-' thedate(5:6) '-' thedate(7:8) ' 00:00:00 +0:00'];
  attribute.time.axis = 'T';

  attribute.range.long_name = 'Range from lidar';
  attribute.range.units = 'm';
  attribute.range.axis = 'Z';

  attribute.beta.units = 'sr-1 m-1' ;
  attribute.beta.units_html = 'sr<sup>-1</sup> m<sup>-1</sup>' ;
  attribute.beta.long_name = 'Attenuated backscatter coefficient' ;
  attribute.beta.comment = 'Speckle noise has been removed, but note that when speckle noise is strong its removal reduces the effective sensitivity of the instrument.';
 
  attribute.global.institution = attribute.global.Copyright_Owner;
  attribute.global.location = 'Potenza, Italy';
  attribute.global.title = 'Potenza CT25K Ceilometer';
  attribute.global.source = 'CT25K Ceilometer';
  attribute.global.history = ['Original data has been averaged to 30 seconds and a calibration factor of 2.5 applied.'];
  attribute.global.day = str2double(thedate(7:8));
  attribute.global.month = str2double(thedate(5:6));
  attribute.global.year = str2double(thedate(1:4));
end

% standardise the units here
[data, attribute] = standardise_units(data, attribute);

% Calculate std of velocity estimate from SNR for halo doppler lidar - not required for 
%   latest version of dopplerlidar2nc code
% Would like to extend this to other doppler lidars
if strcmp(instrument,'halo-doppler-lidar')

  if ~isfield(data,'v_error')

    % Correct lidar parameters!
    switch lower(site)
     case {'repartee','malmi'}
      data.lens_diameter = 0.08;
      data.bandwidth = 3e7;
      data.prf = 20000;
     case 'chilbolton'
      data.bandwidth = 5e7;
     case 'actual'
      data.num_samples_gate = 6;
      data.bandwidth = 5e7;
     otherwise
      data.bandwidth = 5e7;
    end
  
    attribute.bandwidth = create_attributes({},'Bandwidth', 'Hz');
  
    % First 3 gates are unusable
    data.beta(:,1:3) = NaN;
    data.v(:,1:3) = NaN;
 
    [data, attribute] = calculate_dl_SNR(data, attribute,lower(site));
  end
end

function newstr = clean_up_string(oldstr)
newstr = num2str(oldstr);
if length(newstr) > 1
  if newstr(end-1) == '\' & newstr(end) == '0'
    newstr = deblank(newstr(1:end-2));
  end
end

function index = despeckle(data);
[a,b]=size(data);
index=find(~isfinite(data(1:a-2,2:b-1)) & ~isfinite(data(3:a,2:b-1)) & ~isfinite(data(2:a-1,1:b-2)) & ~isfinite(data(2:a-1,3:b)));

function param_out = remove_inner_pixels(param, index, type, value)

if nargin<3
  type='line';
  value = NaN;
end
if nargin < 4
  value = NaN;
end

switch type
  case 'line'
   param_inner = param(2:end-1,:);
   param_inner(index) = value;
   param_out = param;
   param_out(2:end-1,:) = param_inner;
 case 'despeckle'  
   param_inner = param(2:end-1,2:end-1);
   param_inner(index) = value;
   param_out = param;
   param_out(2:end-1,2:end-1) = param_inner;
end


function [data, attribute] = process_chm15k(data,attribute,thedate,calibration_factor);
  
%  data.lidar_beam_divergence: 0.7500
%  data.lidar_telescope_field_of_view: 0.6600

% Need new time, seconds since 1970
  epoch_time = etime([str2double(thedate(1:4)) str2double(thedate(5:6)) str2double(thedate(7:8)) 0 0 0],[1970 1 1 0 0 0 ]);

  if data.time(1) > epoch_time
    % assume time is in units: 'seconds since 1904-01-01 00:00:00.000 00:00'
    epoch_time = epoch_time + 2082844815;
  end
  
  data.time = (data.time - epoch_time)./3600;   

  % beta_raw is in photon counts. Need to do range correction and noise
  % subtraction, then calibrate..   
  smoothbeta = nanrunmean(nanrunmean(data.beta_raw,7,1),7,2);

  mask = zeros(size(smoothbeta));
  mask(data.beta_raw>5) = 1; % definite signal
  mask(smoothbeta>0.5 & data.beta_raw > 3 & data.beta_raw<=5) = 2; %likely signal
  mask(data.beta_raw > 1 & ones(length(data.time),1)*data.range' < 500) = 3; %possible aerosol signal

  data.beta = data.beta_raw .* (ones(length(data.time),1)*data.range'.^2);
  data.beta(find(mask<1)) = NaN;
  
  % overlap function
  load(['/home/radar/matlab/cloudnet/datafun/mpl_overlap_mace_head.mat']);
  overlap = interp1(overlap_range, overlap, data.range);
  data.beta = data.beta./(ones(length(data.time),1)*overlap');
  
  % calibrate
  data.beta = data.beta .* calibration_factor;

  %despeckle
  index = despeckle(data.beta);
  data.beta = remove_inner_pixels(data.beta, index,'despeckle');
  
  
  
%    (ones(length(time),1)*data.range.^2)
%  data.beta = data.beta.*5e-7;
  
  % Write attributes
  attribute.time.units = ['hours since ' sprintf('%04d-%02d-%02d', str2double(thedate(1:4)), str2double(thedate(5:6)), str2double(thedate(7:8))) ' 00:00:00 +00:00'] ;
  attribute.time.long_name = 'Decimal hours from midnight UTC to the middle of each ray' ;   
  attribute.time.axis = 'T';
  
  attribute.beta.units = 'sr-1 m-1' ;
  attribute.beta.units_html = 'sr<sup>-1</sup> m<sup>-1</sup>' ;
  attribute.beta.long_name = 'Attenuated backscatter coefficient' ;
  attribute.beta.missing_value = 0 ;
  attribute.beta.FillValue_ = 0 ;
  attribute.beta.calibration_factor_applied =  calibration_factor;
  
  attribute.latitude.standard_name = 'latitude';
  attribute.longitude.standard_name = 'longitude';    
  attribute.altitude.units = 'm';
  attribute.altitude.long_name = 'Height of instrument above mean sea level';
  
  %    attribute.elev.units = 'degrees';
  %    attribute.elev.long_name = 'Elevation above horizon';
  %    attribute.elev.missing_value = -999;
  %    attribute.elev.FillValue_ = -999;
  
  %    attribute.beta.comment = 'This variable should be multiplied by the value in the scale_factor attribute. Speckle noise has been removed, but note that when speckle noise is strong its removal reduces the effective sensitivity of the instrument.' ;

  if ~isfield(attribute.global, 'history')
    attribute.global.history = 'unknown';
  end  
  
